<?php
namespace Teknisa\CMP\Services;

use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class ApprovalWorkflowService {
    
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $newCode, $environment){
        $this->entityManager = $entityManager;
        $this->connection    = $entityManager->getConnection();
        $this->newCode       = $newCode;
        $this->environment   = $environment;
    }
    
    public function generateCode($newCode){
        $code = $this->environment->getNrOrgTrab();
        $codeAprovacao = $this->newCode->getCodeByTableName($newCode, $code);
        return $codeAprovacao;
    }
    
    public function verificaAlcada($UNIDADE_DESTINO) {
        try {
            $sql = "SELECT COUNT(*) FILALCADAS FROM FILIALCADA WHERE CDFILIAL = :P_CDFILDEST";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_CDFILDEST" => $UNIDADE_DESTINO
                )
            );
            return $std->fetchAll()[0];
        }
        catch(\Exception $e) {
            throw $e;
        }
    }

    public function findDemandaAF($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA) {
        try {
            $sql = "SELECT CDFILDEST, CDFILLANC, NRAUTOFORN, CDFORNECED, DTAUTOFORN, VRTOTALAF, IDAUTOFORNEX, IDTIPOLANCAF, NVL(IDBAIXAAF, 0) IDBAIXAAF, NVL(IDSITUATUAAF, 0) IDSITUATUAAF, IDAFFORALIM, DECODE(CDFILDEST, CDFILLANC, 'UNIDADE', 'MATRIZ') AS LANCAMENTO, CDOPERLANCAF, CDOPERADOR AS COMPRADOR, IDSITAFALCADA , NRCONFIGALCADA FROM AUTOFORNE WHERE CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRAUTOFORN = :P_NRAUTOFORN";
            $std = $this->connection->prepare($sql);
            $std->execute(array(
                    "P_CDFILDEST"      => $UNIDADE_DESTINO,
                    "P_CDFILLANC"      => $UNIDADE_LANC,
                    "P_NRAUTOFORN"     => $NRDEMANDA
                ));
            return $std->fetchAll()[0];
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }
    
    public function updateDemandaAF($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA, $IDTIPOLANC, $SITUACAO, $NRCONFIGALCADA, $baixaAtual)
    {
        try {
            $sql = "UPDATE AUTOFORNE SET IDSITAFALCADA = :P_SITUACAO, NRCONFIGALCADA = :P_NRCONFIGALCADA, IDBAIXAAF = DECODE(:P_SITUACAO, 'PENDENTE', 6, :P_BAIXA_ATUAL) WHERE CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRAUTOFORN = :P_NRAUTOFORN";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_CDFILDEST"      => $UNIDADE_DESTINO,
                    "P_CDFILLANC"      => $UNIDADE_LANC,
                    "P_NRCONFIGALCADA" => $NRCONFIGALCADA,
                    "P_SITUACAO"       => $SITUACAO,
                    "P_NRAUTOFORN"     => $NRDEMANDA,
                    "P_BAIXA_ATUAL"    => $baixaAtual,
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getBaixaAtualAF($NRDEMANDA)
    {
        try {
            $select = "SELECT IDBAIXAAF FROM AUTOFORNE WHERE NRAUTOFORN = :P_NRAUTOFORN";
            $std = $this->connection->prepare($select);
            $std->execute(
                array(
                    "P_NRAUTOFORN"    => $NRDEMANDA,
                )
            );
            
            return $std->fetch();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findAlcadaLancamento($UNIDADE_DESTINO, $UNIDADE_LANC, $stForaOrc, $NRDEMANDA, $LANCAMENTO, $NMITMORCA, $CDDESPPREV){
        try {
            $select = "SELECT C.NRCONFIGALCADA AS NRCONFIGALCADA FROM AUTOFORNE A, CONFIGALCADA C, FILIALCADA F WHERE A.CDFILDEST = :P_CDFILDEST AND A.CDFILLANC = :P_CDFILLANC AND A.NRAUTOFORN = :P_NRAUTOFORN AND C.NRCONFIGALCADA = F.NRCONFIGALCADA AND F.CDFILIAL = A.CDFILDEST AND ((C.IDLANCAFEXTRA = A.IDAUTOFORNEX) OR (C.IDLANCAFEXTRA = 'AMBOS')) AND ((C.IDTIPOAF = A.IDTIPOLANCAF) OR (C.IDTIPOAF = 'AMBOS')) AND ((C.IDTPLANC = :P_LANCAMENTO) OR (C.IDTPLANC = 'AMBOS')) AND ((C.IDFORAORC = :P_FORAORCAMENTO) OR (:P_FORAORCAMENTO = 'N')) AND ((IDTIPORCAOP = :P_IDTIPORCAOP AND :P_CDDESPPREV = 'T') OR (IDTIPORCAOP = :P_IDTIPORCAOP AND CDDESPPREV = :P_CDDESPPREV))";
            $std = $this->connection->prepare($select);
            $std->execute(
                array(
                    "P_CDFILDEST"     => $UNIDADE_DESTINO,
                    "P_CDFILLANC"     => $UNIDADE_LANC,
                    "P_NRAUTOFORN"    => $NRDEMANDA,
                    "P_LANCAMENTO"    => $LANCAMENTO,
                    "P_IDTIPORCAOP"   => $NMITMORCA,
                    "P_CDDESPPREV"    => $CDDESPPREV,
                    "P_FORAORCAMENTO" => $stForaOrc == 'S' ? 'N' : 'S'
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function totalDiaAF($params) {
        try {
            $select = "SELECT SUM(VRTOTALAF) AS TOTAL FROM AUTOFORNE A, ITEMAUTF I WHERE A.CDFILDEST = :P_CDFILDEST AND A.NRAUTOFORN <> :P_NRAUTOFORN AND A.CDFILDEST = I.CDFILDEST AND A.NRAUTOFORN = I.NRAUTOFORN AND A.CDFILDEST||A.CDFORNECED||I.CDPRODUTO||I.DTPROGENTAF IN (SELECT ITEMAUTF.CDFILDEST||AUTOFORNE.CDFORNECED||ITEMAUTF.CDPRODUTO||ITEMAUTF.DTPROGENTAF FROM AUTOFORNE, ITEMAUTF WHERE AUTOFORNE.CDFILDEST = :P_CDFILDEST AND AUTOFORNE.CDFILLANC = :P_CDFILLANC AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN AND AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST AND AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN)";
            $std = $this->connection->prepare($select);
            $std->execute(
                array(
                    "P_CDFILDEST"     => $params['CDFILDEST'],
                    "P_CDFILLANC"     => $params['CDFILLANC'],
                    "P_NRAUTOFORN"    => $params['NRDEMANDA'],
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consisteOrcamento($NRDEMANDA) {
        try {
            $sql = "SELECT * FROM CONSISTORCAMENTO WHERE NRDEMANDA = :P_NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_NRDEMANDA" => $NRDEMANDA
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function valida_Oper($CDOPERLANC, $UNIDADE_DESTINO, $NRCONFIGALCADA) {
        try {
            $sql = "SELECT CDOPERADOR FROM ALCADAGERFILI WHERE NRCONFIGALCADA = :P_NRCONFIGALCADA AND CDFILIAL = :P_CDFILDEST AND (CDOPERADOR = :P_CDOPERLANC) UNION ALL SELECT CDOPERADOR FROM ALCADAGEROPER WHERE NRCONFIGALCADA = :P_NRCONFIGALCADA AND CDFILIAL = :P_CDFILDEST AND (CDOPERRELACAP = :P_CDOPERLANC)";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_CDFILDEST"      => $UNIDADE_DESTINO,
                    "P_NRCONFIGALCADA" => $NRCONFIGALCADA,
                    "P_CDOPERLANC"     => $CDOPERLANC
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    
    }
    
    public function executeCONSISTE_ORCAMENTO($UNIDADE_DESTINO, $UNIDADE_LANC, $NRDEMANDA){
        try{
            $connection = $this->entityManager->getConnection();
            $procedure  = new StoredProcedure($connection, 'CONSISTE_ORCAMENTO_GOP');
            $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT,  $UNIDADE_DESTINO,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,  $UNIDADE_LANC,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT,  $NRDEMANDA, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSISTE',   Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function findAprovador($NRCONFIGALCADA, $VRAPROVCMP, $UNIDADE_DESTINO) {
        try {
            $sql = "SELECT NRALCADAGERFILI, CDOPERADOR, VRAPROVCMP, (SELECT MIN(B.VRAPROVCMP) FROM ALCADAGERFILI B WHERE B.CDFILIAL = ALCADAGERFILI.CDFILIAL AND B.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND B.VRAPROVCMP > ALCADAGERFILI.VRAPROVCMP) VRAPROVCMPMAX FROM ALCADAGERFILI WHERE CDFILIAL = :P_CDFILIAL AND NRCONFIGALCADA = :P_NRCONFIGALCADA AND VRAPROVCMP = (SELECT MIN(C.VRAPROVCMP) FROM ALCADAGERFILI C WHERE C.CDFILIAL = ALCADAGERFILI.CDFILIAL AND C.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND ((C.VRAPROVCMP > :P_VRAPROVCMP) OR NVL(:P_VRAPROVCMP, 'T') = 'T')) ORDER BY VRAPROVCMP";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_CDFILIAL"       => $UNIDADE_DESTINO,
                    "P_NRCONFIGALCADA" => $NRCONFIGALCADA,
                    "P_VRAPROVCMP"     => $VRAPROVCMP
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function inserirRegistroAprov($UNIDADE_DESTINO, $UNIDADE_LANC, $NRCONFIGALCADA, $NRDEMANDA, $CDOPERADOR, $IDTPDEMANDA, $IDCLASSIFIC, $IDOPERACAOALC, $SITUACAO) {
        try {
            $sql = "INSERT INTO APROVALCADA ( NRAPROVALCADA, NRCONFIGALCADA, CDFILDEST, CDFILLANC, NRDEMANDA, CDOPERADOR, IDTPDEMANDA, IDCLASSIFIC, IDOPERACAOALC, IDAPROVADO, DTINCLUSAO ) VALUES ( :P_NEWNRAPROVALCADA, :P_NRCONFIGALCADA, :P_CDFILDEST, :P_CDFILLANC, :P_NRDEMANDA, :P_CDOPERADOR, :P_IDTPDEMANDA, :P_IDCLASSIFIC, :P_IDOPERACAOALC, :P_SITUACAO, SYSDATE )";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "P_NEWNRAPROVALCADA" => (float)$this->generateCode('APROVALCADA'),
                    "P_NRCONFIGALCADA"   => $NRCONFIGALCADA,
                    "P_CDFILDEST"        => $UNIDADE_DESTINO,
                    "P_CDFILLANC"        => $UNIDADE_LANC,
                    "P_NRDEMANDA"        => $NRDEMANDA,
                    "P_CDOPERADOR"       => $CDOPERADOR,
                    "P_IDTPDEMANDA"      => $IDTPDEMANDA,
                    "P_IDCLASSIFIC"      => $IDCLASSIFIC,
                    "P_IDOPERACAOALC"    => $IDOPERACAOALC,
                    "P_SITUACAO"         => $SITUACAO,
                )
            );
        }
        catch(\Exception $e) {
            throw $e;
        }
    }

    public function getOperAddress($NRALCADAGERFILI) {
        try {
            $sql = "SELECT A.CDOPERADOR, G.NMOPERADOR, G.DSEMAILOPER FROM ALCADAGERFILI A, OPERADOR G WHERE A.NRALCADAGERFILI = :P_NRALCADAGERFILI AND A.CDOPERADOR = G.CDOPERADOR UNION SELECT O.CDOPERRELACAP CDOPERADOR, R.NMOPERADOR, R.DSEMAILOPER FROM ALCADAGERFILI A, ALCADAGEROPER O, OPERADOR R WHERE A.NRALCADAGERFILI = :P_NRALCADAGERFILI AND A.NRCONFIGALCADA = O.NRCONFIGALCADA AND A.CDFILIAL = O.CDFILIAL AND A.CDOPERADOR = O.CDOPERADOR AND O.CDOPERRELACAP = R.CDOPERADOR";
            $std = $this->connection->prepare($sql);
            $std->execute(array(
                    "P_NRALCADAGERFILI" => $NRALCADAGERFILI
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }
    
    public function getTipoOrcamento($CDFILDEST, $CDFILLANC, $NRDEMANDA) {
        try {
            $sql = "SELECT DISTINCT IDORIGEMCONS NMITMORCA, NVL(CDDESPPREV, 'T') CDDESPPREV FROM CONSISTORCAMENTO WHERE CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRDEMANDA = :P_NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'P_CDFILDEST' => $CDFILDEST,
                    'P_CDFILLANC' => $CDFILLANC,
                    'P_NRDEMANDA' => $NRDEMANDA
                )
            );
            return $std->fetchAll()[0];
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDemandaDespesa($CDFILDEST, $CDFILLANC, $NRDEMANDA) {
        try {
            $sql = "SELECT MAX(CDDESPPREV) CDDESPPREV, 'DESPESA' NMITMORCA FROM PRODDESPPREV, ITEMAUTF WHERE ITEMAUTF.CDPRODUTO LIKE PRODDESPPREV.CDPRODUTO||'%' AND CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRAUTOFORN = :P_NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                    array(
                        'P_CDFILDEST' => $CDFILDEST,
                        'P_CDFILLANC' => $CDFILLANC,
                        'P_NRDEMANDA' => $NRDEMANDA
                    )
                );
            return $std->fetchAll()[0];
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDemandaConsumo($CDFILDEST, $CDFILLANC, $NRDEMANDA) {
        try {
            $sql = "SELECT MAX(PE.NRPERCITEMGOP) NRPERCITEMGOP, 'CONSUMO' NMITMORCA, 'T' CDDESPPREV  FROM PERCITEMDETALGOP PE, ITEMAUTF WHERE PE.NRPERCITEMGOP = '2' AND ITEMAUTF.CDPRODUTO LIKE PE.CDPRODUTO||'%' AND CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRAUTOFORN = :P_NRDEMANDA ORDER BY PE.CDPRODUTO ASC";
            $std = $this->connection->prepare($sql);
            $std->execute(
                    array(
                        'P_CDFILDEST' => $CDFILDEST,
                        'P_CDFILLANC' => $CDFILLANC,
                        'P_NRDEMANDA' => $NRDEMANDA
                    )
                );
            return $std->fetchAll()[0];
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getNMFILIAL($UNIDADE_DESTINO) {
        try {
            $sql = "SELECT NMFILIAL FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL";
            $std = $this->connection->prepare($sql);
            $std->execute(array(
                    "P_CDFILIAL" => $UNIDADE_DESTINO
                ));
            return $std->fetchAll()[0]['NMFILIAL'];
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }
    
    public function getNMALCADA($NRCONFIGALCADA) {
        try {
            $sql = "SELECT NMALCADA FROM CONFIGALCADA WHERE NRCONFIGALCADA = :P_NRCONFIGALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(array(
                    "P_NRCONFIGALCADA" => $NRCONFIGALCADA
                ));
            return $std->fetchAll()[0]['NMALCADA'];
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }
    
    public function getOperLancamento($CDOPERLANCAF) {
        try {
            $sql = "SELECT NMOPERADOR, DSEMAILOPER FROM OPERADOR WHERE CDOPERADOR = :P_CDOPERLANC";
            $std = $this->connection->prepare($sql);
            $std->execute (
                array (
                    'P_CDOPERLANC'     => $CDOPERLANCAF
                )
            );
            $result = $std->fetchAll();
            if(!empty($result)) {
                return $result[0];
            }
            else {
                return array (
                    'NMOPERADOR'  => '',
                    'DSEMAILOPER' => ''
                );
            }
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }
    
    public function verificaRegistroAprov($cdfildest, $cdfillanc, $nrdemanda) {
        try {
            $sql = "SELECT * FROM APROVALCADA WHERE CDFILDEST = :CDFILDEST AND CDFILLANC = :CDFILLANC AND NRDEMANDA = :NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "CDFILDEST" => $cdfildest,
                    "CDFILLANC" => $cdfillanc,
                    "NRDEMANDA" => $nrdemanda,
                )
            );
            return $std->fetchAll();
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizarRegistroAprov($nrconfigalcada, $cdfildest, $cdfillanc, $nrdemanda, $cdoperador, $situacao) {
        try {
            $sql = "UPDATE APROVALCADA SET IDAPROVADO = :IDAPROVADO WHERE NRCONFIGALCADA = :NRCONFIGALCADA AND CDFILDEST = :CDFILDEST AND CDFILLANC = :CDFILLANC AND NRDEMANDA = :NRDEMANDA AND CDOPERADOR = :CDOPERADOR";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "NRCONFIGALCADA" => $nrconfigalcada,
                    "CDFILDEST" => $cdfildest,
                    "CDFILLANC" => $cdfillanc,
                    "NRDEMANDA" => $nrdemanda,
                    "CDOPERADOR" => $cdoperador,
                    "IDAPROVADO" => $situacao
                )
            );
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findCodigoProdutoOrcamento($params)
    {
        try {
            $select = "SELECT MAX(PROD.CDARVPROD) CDARVPROD
                        FROM   ORCAMENTOGOP O
                               INNER JOIN ITEMORCAGOP I
                                       ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                          AND I.IDTIPOORCAOP IN ( 'CONSUMO', 'DESPENTRADA',
                                                                  'DESPDIVERSA' )
                               --  AND I.NRORG = O.NRORG
                               INNER JOIN APROVAORCAGOP A
                                       ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                               --  AND A.NRORG = O.NRORG
                               INNER JOIN ITORCAMESGOP IM
                                       ON IM.NRITEMORCAOP = I.NRITEMORCAOP
                               --   AND IM.NRORG = I.NRORG
                               INNER JOIN ITDETALORCAMESGOP IDP
                                       ON IDP.NRITORCAMESGOP = IM.NRITORCAMESGOP
                               -- AND IDP.NRORG = IM.NRORG
                               INNER JOIN PRODUTO PROD
                                       ON PROD.CDPRODUTO = IDP.CDPRODUTO
                        --    AND PROD.NRORG = IDP.NRORG
                        WHERE  O.CDFILIAL = :P_CDFILDEST
                               AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                               AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                               AND O.NRANOORCAGOP = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'YYYY')
                               --'2019'--TO_CHAR(:P_DTPROGENTAF,'YYYY')             
                               AND TO_CHAR(IM.DTORCAMENTO, 'MM') = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'MM')
                               AND IDP.VRORCAMENTO IS NOT NULL
                               AND EXISTS(SELECT 1
                                          FROM   ITEMAUTF,
                                                 PRODUTO
                                          WHERE  ITEMAUTF.CDFILDEST = :P_CDFILDEST
                                                 AND ITEMAUTF.CDFILLANC = :P_CDFILLANC
                                                 AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN
                                                 AND ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO
                                                 AND PRODUTO.CDARVPROD LIKE PROD.CDARVPROD
                                                                            ||'%')";
            $std = $this->connection->prepare($select);
            $std->execute(
                array(
                    "P_IDPERIODOORCA"    => $params['IDPERIODOORCA'],
                    "P_NRPERIODOORCA"    => $params['NRPERIODOORCA'],
                    "P_DTPROGENTAF"      => $params['DTPROGENTAF'],
                    "P_CDFILDEST"        => $params['CDFILDEST'],
                    "P_CDFILLANC"        => $params['CDFILLANC'],
                    "P_NRAUTOFORN"       => $params['NRDEMANDA'],
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getTotalPeriodoAF($params)
    {
        try {
            $select = "SELECT
                            NVL(SUM(ITEMAUTF.VRPREUNIAF * ITEMAUTF.QTPROGENTAF), 0) VR_TOTAL_AF
                        FROM
                            ITEMAUTF,
                            AUTOFORNE,
                            PRODUTO
                        WHERE
                            AUTOFORNE.CDFILDEST                        = ITEMAUTF.CDFILDEST
                        AND AUTOFORNE.CDFILLANC                      = ITEMAUTF.CDFILLANC
                        AND AUTOFORNE.NRAUTOFORN                     = ITEMAUTF.NRAUTOFORN
                        AND TO_CHAR(ITEMAUTF.DTPROGENTAF, 'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTPROGENTAF),
                            'MM/YYYY')
                        AND NVL(ITEMAUTF.IDBAIXITMAF, '1')    IN ( '1', '2' )
                        AND NVL(AUTOFORNE.IDBAIXAAF, '1') NOT IN ( '3', '4', '5' )
                        AND ITEMAUTF.CDFILDEST                 = :P_CDFILDEST
                        AND ITEMAUTF.NRAUTOFORN               <> :P_NRAUTOFORN
                        AND ITEMAUTF.CDPRODUTO                 = PRODUTO.CDPRODUTO
                        AND PRODUTO.CDARVPROD LIKE :P_CDARVPROD
                            ||'%' 
                        AND EXISTS(SELECT  1
                            FROM APROVALCADA
                            WHERE APROVALCADA.CDFILDEST = :P_CDFILDEST   
                            AND APROVALCADA.NRDEMANDA = AUTOFORNE.NRAUTOFORN
                            AND NVL(IDAPROVADO, 'A') = 'APROVADO') ";
            $std = $this->connection->prepare($select);
            $std->execute(
                array(
                    "P_DTPROGENTAF" => $params['DTPROGENTAF'],
                    "P_CDFILDEST"   => $params['CDFILDEST'],
                    "P_NRAUTOFORN"  => $params['NRDEMANDA'],
                    "P_CDARVPROD"   => $params['CDARVPROD'],
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findDataOrcamento($params)
    {
        try {
            $query = "SELECT V_DATAINI, ADD_MONTHS(V_DATAINI,1)-1 V_DATAFIM 
                        FROM (
                            SELECT TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                         THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                         ELSE LPAD(:P_DIA,2,'0')
                                    END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY') V_DATAINI
                               FROM DATA P
                              WHERE :P_DATA BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                ELSE LPAD(:P_DIA,2,'0')
                                                           END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                                               AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                           THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                           ELSE LPAD(:P_DIA,2,'0')
                                                                      END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'),1)-1
                              GROUP BY TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                 THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                 ELSE LPAD(:P_DIA,2,'0')
                                            END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'))";
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "P_DIA"       => $params['DIA'],
                    "P_DATA"      => $params['DATA'],
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findDiaInicioOrcamento($cdFilialDestino)
    {
        try {
            $query = "SELECT IDCONSAFORCFIL, VRPEORCBLOQFIL, NVL(P.IDVERIORCFILI,'NAO_VALIDA') IDVERIORCFILI,
                            NVL(F.NRDIAINIORCAGOP,PO.NRDIAINIORCA) DIAINIORCA, NVL(PO.CDTIPOCUSTO,'03') CDTIPOCUSTO
                        FROM FILIAL F, PARASUPRI P, PARAMORCAOPERAC PO
                        WHERE F.CDFILIAL = :P_CDFILDEST
                        AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)
                        AND F.NRORG    = :P_NRORG
                        AND PO.NRORG   = :P_NRORG
                        AND F.NRORG    = P.NRORG";
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "P_CDFILDEST" => $cdFilialDestino,
                    "P_NRORG"     => $this->environment->getNrOrg(),
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findPeriodoOrcamento($params)
    {
        try {
            $query = "SELECT MAX(NRPERIODO) V_NRPERIODOORCAGOP, MAX(IDPERIODOORCA) V_IDPERIODOORCA
                         FROM PERIODOORCAGOP P
                        WHERE P.NRORG = :P_NRORG
                          AND TO_DATE(:V_DATAINI) BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                             THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                             ELSE LPAD(:DIAINIORCA,2,'0')
                                                       END || TO_CHAR(P.DTINIPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY')
                                            AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                                        THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                                        ELSE LPAD(:DIAINIORCA,2,'0')
                                                                   END || TO_CHAR(P.DTFIMPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY'),1)-1";
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "DIAINIORCA" => $params['DIAINIORCA'],
                    "V_DATAINI"  => $params['V_DATAINI'],
                    "P_NRORG"    => $this->environment->getNrOrg(),
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findDataEntregaAF($params)
    {
        try {
            $query = "SELECT DTPROGENTAF
                       FROM ITEMAUTF AF
                      WHERE AF.CDFILDEST  = :P_CDFILDEST
                        AND AF.CDFILLANC  = :P_CDFILLANC
                        AND AF.NRAUTOFORN = :P_NRDEMANDA
                        AND AF.NRORG      = :P_NRORG
                      GROUP BY DTPROGENTAF";
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "P_CDFILDEST" => $params['CDFILDEST'],
                    "P_CDFILLANC" => $params['CDFILLANC'],
                    "P_NRDEMANDA" => $params['NRDEMANDA'],
                    "P_NRORG"     => $this->environment->getNrOrg(),
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findFilialAlcada($params)
    {
        try {
            $query = "SELECT
                            NRFILIALCADA,
                            NRCONFIGALCADA,
                            CDFILIAL,
                            IDCONSISTVRAPROV
                        FROM
                            FILIALCADA
                        WHERE
                            NRCONFIGALCADA = :P_NRCONFIGALCADA
                            AND CDFILIAL = :P_CDFILDEST
                            AND NRORG = :P_NRORG";
            $std = $this->connection->prepare($query);
            $std->execute(
                array(
                    "P_CDFILDEST"      => $params['CDFILDEST'],
                    "P_NRCONFIGALCADA" => $params['NRCONFIGALCADA'],
                    "P_NRORG"          => $this->environment->getNrOrg(),
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deletarAprovacoesExistentes($nrdemanda) {
        try {
            $sql = "DELETE FROM APROVALCADA WHERE NRDEMANDA = :NRDEMANDA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    "NRDEMANDA" => $nrdemanda,
                )
            );
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findAprovacaoRecusada($row)
    {
        $query = "SELECT
                        APROVALCADA.NRDEMANDA AS NRAUTOFORN,
                        APROVALCADA.DTAPROVACAO,
                        APROVALCADA.DTINCLUSAO,
                        APROVALCADA.IDAPROVADO
                    FROM
                        APROVALCADA
                    WHERE
                        APROVALCADA.CDFILDEST = :P_CDFILDEST
                        AND APROVALCADA.CDFILLANC = :P_CDFILLANC
                        AND APROVALCADA.NRDEMANDA = :P_NRAUTOFORN
                        AND APROVALCADA.NRORG = :P_NRORG
                        AND ROWNUM = 1
                    ORDER BY APROVALCADA.DTAPROVACAO DESC";
        
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILDEST" => $row['CDFILDEST'],
            "P_CDFILLANC" => $row['CDFILLANC'],
            "P_NRAUTOFORN" => $row['NRAUTOFORN'],
            "P_NRORG" => $this->environment->getNrOrg(),
        ));
        
        return $std->fetch();
    }
}